from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from app.models import Event, Request, User
from app.forms import *


def home(request):
    return render(request, "home.html")


def displayEvent(request):
    return render(request, "displayEvent.html", {"events": Event.objects.all()})


@login_required
def createEvent(request):
    if request.method == "POST":
        form = EventForm(request.POST)
        if form.is_valid():
            event_save = form.save(commit=False)
            event_save.creator = request.user
            event_save.comment = ""
            event_save.save()
            return render(request, "displayEvent.html", {"events": Event.objects.all()})
    else:
        form = EventForm()
    return render(request, "createEvent.html", {"form": form})


@login_required
def modifyEvent(request, id):
    event = Event.objects.get(id=id)
    if request.method == "POST":
        form = EventForm(request.POST, instance=event)
        if form.is_valid():
            form.save()
            return render(request, "displayEvent.html", {"events": Event.objects.all()})
    else:
        form = EventForm(instance=event)
    return render(request, "createEvent.html", {"form": form})


@login_required
def deleteEvent(request, id):
    event = Event.objects.get(id=id)
    event.delete()
    return redirect("evenement_liste")


@login_required
def requests(request):
    #clubs = LdapEleve.objects.get(uid=request.user.username).get_clubs()
    clubs = LdapEleve.objects.get(uid="valliet").get_clubs()
    requests = Request.objects.filter(club__in=[club.cn for club in clubs])
    for req in requests:
        req.club = LdapClub.objects.get(cn=req.club)
        req.authorized = sanitize_user_input(req.authorized)
    return render(request, "request.html", {"requests": requests})


@login_required
def pendingRequest(request):
    requests = Request.objects.filter(state="ATT_VAL")
    for req in requests:
        req.club = LdapClub.objects.get(cn=req.club)
        req.authorized = sanitize_user_input(req.authorized)
    return render(
        request,
        "pendingRequest.html",
        {"pendingRequests": requests},
    )


# transforme une string en liste de LdapEleve
def sanitize_user_input(string: str):
    ret = []
    for username in string.split(","):
        if LdapEleve.objects.filter(uid=username).exists():
            ret.append(LdapEleve.objects.get(uid=username))
    return ret


@login_required
def createRequest(request, id):
    if request.method == "POST":
        form = RequestForm(request.POST)
        if form.is_valid():
            request_save = form.save(commit=False)
            request_save.creator = request.user
            request_save.comment = ""
            request_save.state = "ATT_VAL"
            request_save.authorized = sanitize_user_input(request.POST["authorized"])
            request_save.save()
            return render(request, "request.html")
    else:
        event = Event.objects.get(id=id)
        eleve = LdapEleve.objects.get(uid=request.user.username)
        form = RequestForm()
        form.fields["event"].initial = event
        form.fields["club"].widget.choices = request.user.get_board_clubs()
        form.fields["contact_phone"].initial = eleve.mobile
        form.fields["contact_mail"].initial = eleve.mail
    return render(request, "createRequest.html", {"form": form})


# TODO -> Quand une requête est erronée, le champ club se reset
# faire en sorte que le champ club ne se reset pas
@login_required
def modifyRequest(request, id):
    req = Request.objects.get(id=id)
    if request.method == "POST":
        form = RequestForm(request.POST, instance=req)
        form.fields["club"].widget = TextInput(attrs={"readonly": "True", "value": req.club})
        if form.is_valid():
            form.save()
            return redirect("requete_validation")
    else:
        form = RequestForm(instance=req)
        form.fields["club"].widget = TextInput(attrs={"readonly": "True", "value": req.club})
    return render(request, "createRequest.html", {"form": form})


@login_required
def refuseRequest(request, id):
    req = Request.objects.get(id=id)
    # if request.user.type == "RC" or request.user.type == "ADM":
    req.state = "REF"
    req.save()
    return redirect("requete_validation")


@login_required
def acceptRequest(request, id):
    req = Request.objects.get(id=id)
    if (
        ((req.state == "ATT_VAL") & (request.user.type == "RC"))
        | ((req.state == "VAL_RC") & (request.user.type == "ADM"))
        | ((req.state == "VAL_ADM") & (request.user.type == "STI"))
    ):
        req.state = req.state + 1
        req.save()
    else:
        # TODO: mettre un message pour indiquer qu'un personne n'a pas les droits pour valider la requete
        None
    return redirect("requete_validation")
